@@ -58,4 +58,8 @@ def findfiles(pattern, folder='.'):
 
     # TODO: recursive param with walk() filtering
     rule = re.compile(fnmatch.translate(pattern), re.IGNORECASE)
-    return [os.path.join(folder, name) for name in os.listdir(folder) if rule.match(name)]
\ No newline at end of file
+    return [os.path.join(folder, name) for name in os.listdir(folder) if rule.match(name)]
+
+
+def dos2unix(s):
+    return s.replace('\r\n', '\n')
\ No newline at end of file
