@@ -324,38 +324,61 @@ class ProblemService:
                 print("Detected translation file(s):", *tran_files, sep="\n")
 
             statement_file = os.path.join(problem_folder, f"{problem_code}.md")
-            stock_solution_file = os.path.join(problem_folder, f"{problem_code}.py")
-            stock_solution_file_cpp = os.path.join(problem_folder, f"{problem_code}.cpp")
+            # stock_solution_file = os.path.join(problem_folder, f"{problem_code}.py")
+            # stock_solution_file_cpp = os.path.join(problem_folder, f"{problem_code}.cpp")
             metadata_file = os.path.join(problem_folder, f"{problem_code}_meta.yaml")
             metadata_file_json = os.path.join(problem_folder, f"{problem_code}_meta.json")
-            test_generator_file = os.path.join(problem_folder, f"{problem_code}_generator.py")
+            # test_generator_file = os.path.join(problem_folder, f"{problem_code}_generator.py")
+            # test_generator_file_cpp = os.path.join(problem_folder, f"{problem_code}_generator.cpp")
             editorial_file = os.path.join(problem_folder, f"{problem_code}_editorial.md")
             testcase_file = os.path.join(problem_folder, f"{problem_code}_testcases.txt")
             testcase_sample_file = os.path.join(problem_folder, f"{problem_code}_testcases_sample.txt")
             tranlation_file = os.path.join(problem_folder, f"{problem_code}_tran_%s.md")
         else:
             statement_file = os.path.join(problem_folder, f"statement.md")
-            stock_solution_file = os.path.join(problem_folder, f"solution.py")
-            stock_solution_file_cpp = os.path.join(problem_folder, f"solution.cpp")
+            # stock_solution_file = os.path.join(problem_folder, f"solution.py")
+            # stock_solution_file_cpp = os.path.join(problem_folder, f"solution.cpp")
+            # stock_solution_file_pas = os.path.join(problem_folder, f"solution.pas")
             metadata_file = os.path.join(problem_folder, f"metadata.yaml")
             metadata_file_json = os.path.join(problem_folder, f"metadata.json")
-            test_generator_file = os.path.join(problem_folder, f"input_generator.py")
+            # test_generator_file = os.path.join(problem_folder, f"input_generator.py")
+            # test_generator_file_cpp = os.path.join(problem_folder, f"input_generator.cpp")
+            # test_generator_file_pas = os.path.join(problem_folder, f"input_generator.pas")
             editorial_file = os.path.join(problem_folder, f"editorial.md")
             testcase_file = os.path.join(problem_folder, f"testcases.txt")
             testcase_sample_file = os.path.join(problem_folder, f"testcases_sample.txt")
             tranlation_file = os.path.join(problem_folder, f"statement_tran_%s.md")
 
+        all_files = sorted(glob.glob(os.path.join(problem_folder, "*.*")))
+        solution_files = []
+        test_generator_files = []
+
+        for file in all_files:
+            filename, file_ext = os.path.splitext(file)
+            if ("solution" in filename or filename.endswith(problem_code)) \
+                and file_ext in [".py", ".pas", ".cpp"]:
+                solution_files.append(file)
+            elif "generator" in filename and file_ext in [".py", ".pas", ".cpp"]:
+                test_generator_files.append(file)
+        # solution_files = sorted(glob.glob(os.path.join(problem_folder,
+        #                                                f"*solution*\.(pas$|py$|cpp$)")))
+        # solution_files += sorted(glob.glob(os.path.join(problem_folder,
+        #                                                 f"{problem_code}\.(pas$|py$|cpp$)")))
+        # test_generator_files = sorted(glob.glob(os.path.join(problem_folder,
+        #                                                      f"*input_generator*\.(pas$|py$|cpp$)")))
+        # test_generator_files += sorted(glob.glob(os.path.join(problem_folder,
+        #                                                       f"{problem_code}_generator*\.(pas$|py$|cpp$)")))
+
         problem_files = {
             "statement": statement_file,
             "translation": tranlation_file,
             "editorial": editorial_file,
             "metadata": metadata_file,
             "metadata_json": metadata_file_json,
-            "stock_solution": stock_solution_file,
-            "stock_solution_cpp": stock_solution_file_cpp,
             "testcases": testcase_file,
             "testcases_sample": testcase_sample_file,
-            "test_generator": test_generator_file
+            "test_generators": test_generator_files,
+            "solutions": solution_files
         }
 
         if not os.path.isfile(metadata_file) and not os.path.isfile(metadata_file_json):
@@ -418,12 +441,10 @@ class ProblemService:
             CLog.error("No metadata detected!")
             return None, None
 
-        if not os.path.isfile(problem_files["stock_solution"]) \
-                and not os.path.isfile(problem_files["stock_solution_cpp"]):
-            CLog.error(f"Main Python solution file `{problem_files['stock_solution']}` "
-                       f"or `{problem_files['stock_solution_cpp']}.cpp` is missing!")
-        if not os.path.isfile(problem_files['test_generator']):
-            CLog.error(f"Testcase input generator file `{problem_files['test_generator']}` is missing!")
+        if not problem_files["solutions"]:
+            CLog.warn(f"Solution file is missing!")
+        if not problem_files['test_generators']:
+            CLog.warn(f"Testcase input generator file is missing!")
         if not os.path.isfile(problem_files['editorial']):
             CLog.warn(f"Editorial file `{problem_files['editorial']}` is missing!")
         if not os.path.isfile(problem_files['testcases']):
@@ -498,7 +519,7 @@ class ProblemService:
 
     @staticmethod
     def create_problem(folder, problem_name, problem_code=None, lang="vi", translations=[], overwrite=False,
-                       tags=[], difficulty=0.0, programming_language='py'):
+                       tags=[], difficulty=0.0, programming_language='py', gen_sample=False):
         problem = Problem()
         problem.name = problem_name
         if not problem_code:
@@ -511,14 +532,21 @@ class ProblemService:
 
         template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')
 
-        with open(os.path.join(template_path, f'editorial.md'),
+        prefix_path = ""
+        if gen_sample:
+            prefix_path = "sample/"
+
+        with open(os.path.join(template_path, f'{prefix_path}editorial.md'),
                   encoding="utf-8") as f:
             problem.editorial = Template(f.read()).render(problem=problem)
 
-        ProblemService._parse_statement_file(os.path.join(template_path, "statement.md"), problem)
+        ProblemService._parse_statement_file(os.path.join(template_path, f"{prefix_path}statement.md"), problem)
         problem.name = problem_name
         problem.code = problem_code
 
+        if not gen_sample:
+            problem.testcases_sample = []
+
         with open(os.path.join(template_path, f'solution.{programming_language}.j2'),
                   encoding="utf-8") as f:
             content = Template(f.read()).render(problem_code=problem_code)
@@ -529,7 +557,7 @@ class ProblemService:
         with open(os.path.join(template_path, f'generator.{programming_language}.j2'),
                   encoding="utf-8") as f:
             content = Template(f.read()).render(problem_code=problem_code)
-            problem.testcase_generator = {"lang": programming_language, "code": content}
+            problem.testcase_generators = [{"lang": programming_language, "code": content}]
 
         for t in translations:
             problem.translations[t] = Problem(name=problem_name)
@@ -672,7 +700,8 @@ class ProblemService:
                     problem.testcases_sample.append(testcase)
 
     @staticmethod
-    def load(problem_folder, load_testcase=False, load_testcases_only=False, force_testcase_format=None):
+    def load(problem_folder, load_testcase=False, load_testcases_only=False,
+             force_testcase_format=None):
         problem = Problem()
         meta, problem_files = ProblemService.check_problem(problem_folder)
         if not meta:
@@ -716,13 +745,6 @@ class ProblemService:
 
             print("src", problem.src_url)
 
-            if os.path.exists(problem_files['stock_solution']):
-                with open(problem_files['stock_solution'], encoding="utf-8") as fi:
-                    problem.solution = fi.read()
-            elif os.path.exists(problem_files['stock_solution_cpp']):
-                with open(problem_files['stock_solution_cpp'], encoding="utf-8") as fi:
-                    problem.solution = fi.read()
-
             if os.path.exists(problem_files["editorial"]):
                 editorial_prob = Problem()
                 ProblemService._parse_statement_file(problem_files["editorial"], editorial_prob)
@@ -742,27 +764,33 @@ class ProblemService:
                         ProblemService._parse_statement_file(lang_statement_file, tran_problem)
                         problem.translations[lang] = tran_problem
 
-            solution_file = os.path.join(problem_folder, f"{problem_files['stock_solution']}")
-            if os.path.isfile(solution_file):
-                idx = solution_file.rfind(".")
-                with open(solution_file, encoding="utf-8") as f:
-                    problem.solution = f.read()
-                    problem.solution_lang = solution_file[idx + 1:]
+            # if os.path.exists(problem_files['stock_solution']):
+            #     with open(problem_files['stock_solution'], encoding="utf-8") as fi:
+            #         problem.solution = fi.read()
+            # elif os.path.exists(problem_files['stock_solution_cpp']):
+            #     with open(problem_files['stock_solution_cpp'], encoding="utf-8") as fi:
+            #         problem.solution = fi.read()
+            #
 
-            additional_solution_files = glob.glob(os.path.join(problem_folder, f"*_solution.*"))
-            for file_path in additional_solution_files:
-                idx = file_path.rfind(".")
-                with open(file_path, encoding="utf-8") as f:
-                    problem.solutions.append({"lang": file_path[idx + 1:],
-                                              "code": f.read()})
-            # print(*problem.solutions, sep="\n")
+            if problem_files['solutions']:
+                for file_path in problem_files['solutions']:
+                    idx = file_path.rfind(".")
+                    with open(file_path, encoding="utf-8") as f:
+                        problem.solutions.append({"lang": file_path[idx + 1:],
+                                                  "code": f.read()})
+                if problem.solutions:
+                    problem.solution = problem.solutions[0]["code"]
+                    problem.solution_lang = problem.solutions[0]["lang"]
+                # print(*problem.solutions, sep="\n")
 
-            file_path = problem_files['test_generator']
-            if os.path.exists(file_path):
-                idx = file_path.rfind(".")
-                with open(file_path, encoding="utf-8") as f:
-                    problem.testcase_generator = {"lang": file_path[idx + 1:], "code": f.read()}
-            # print(problem.testcase_generator)
+            problem.testcase_generators = []
+            if problem_files['test_generators']:
+                for file_path in problem_files['test_generators']:
+                    if os.path.exists(file_path):
+                        idx = file_path.rfind(".")
+                        with open(file_path, encoding="utf-8") as f:
+                            problem.testcase_generators.append({"lang": file_path[idx + 1:], "code": f.read()})
+                    # print(problem.testcase_generator)
 
         if load_testcase:
             testcase_format = problem.testcase_format
@@ -992,81 +1020,48 @@ class ProblemService:
             #     f.write(problem.solution)
             #     f.close()
 
-            if problem.testcase_generator:
-                f = open(os.path.join(problem_folder, f"input_generator.{problem.testcase_generator['lang']}"),
-                         'w', encoding="utf-8", newline='')
-                f.write(problem.testcase_generator["code"])
-                f.close()
+            if problem.testcase_generators:
+                for testcase_generator in problem.testcase_generators:
+                    f = open(os.path.join(problem_folder, f"input_generator.{testcase_generator['lang']}"),
+                             'w', encoding="utf-8", newline='')
+                    f.write(testcase_generator["code"])
+                    f.close()
 
+            solution_files = set()
+            solution_codes = set()
             if problem.solution:
-                f = open(os.path.join(problem_folder, f"solution.{problem.solution_lang}"), 'w',
+                solution_file = f"solution.{problem.solution_lang}"
+                solution_files.add(solution_file)
+                f = open(os.path.join(problem_folder, solution_file), 'w',
                          encoding="utf-8", newline='')
                 f.write(problem.solution)
+                solution_codes.add(problem.solution)
                 f.close()
 
             if problem.solutions:
                 for solution in problem.solutions:
-                    f = open(os.path.join(problem_folder, f"solutions.{solution['lang']}"), 'w',
+                    solution_file = f"solution.{solution['lang']}"
+                    i = 1
+                    while solution_file in solution_files:
+                        solution_file = f"solution{i}.{solution['lang']}"
+                        i += 1
+                    solution_files.add(solution_file)
+
+                    if solution["code"] in solution_codes:
+                        continue
+
+                    solution_codes.add(solution["code"])
+                    f = open(os.path.join(problem_folder, solution_file), 'w',
                              encoding="utf-8", newline='')
                     f.write(solution["code"])
                     f.close()
 
         if save_testcases:
-            testcases: List[TestCase] = []
-            if not problem_code:
-                problem_code = os.path.basename(problem_folder)
-            if problem.testcases:
-                testcases += problem.testcases
-
-            with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
-                template = Template(file_.read())
-                for i, t in enumerate(testcases):
-                    t.name = "Test #%02d" % (i+1)
-                content = template.render(testcases=testcases)
-                testcase_path = os.path.join(problem_folder,  "testcases.txt")
-                f = open(testcase_path, 'w', encoding="utf-8", newline='')
-                f.write(content)
-                f.close()
-                CLog.info(f"Testcases saved to '{testcase_path}'")
-
-            if problem.stock_testcases:
-                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
-                    template = Template(file_.read())
-                    for i, t in enumerate(problem.stock_testcases):
-                        t.name = "Test #%02d" % (i+1)
-                    content = template.render(testcases=problem.stock_testcases)
-                    testcase_path = os.path.join(problem_folder,  "testcases_stock.txt")
-                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
-                    f.write(content)
-                    f.close()
-                    CLog.info(f"Testcases saved to '{testcase_path}'")
-
-            sample_testcases = []
-            if problem.testcases_sample:
-                sample_testcases += problem.testcases_sample
-            if problem.stock_testcases_sample:
-                sample_testcases += problem.stock_testcases_sample
-
-            with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
-                template = Template(file_.read())
-                for i, t in enumerate(problem.testcases_sample):
-                    t.name = "Test #%02d" % (i+1)
-                content = template.render(testcases=problem.testcases_sample)
-                testcase_path = os.path.join(problem_folder, "testcases_sample.txt")
-                f = open(testcase_path, 'w', encoding="utf-8", newline='')
-                f.write(content)
-                f.close()
-
-            # if problem.testcase_format == "themis" and overwrite:
-            #     # clear all themis testcases
-            #     test_folders = glob.glob(os.path.join(problem_folder, "Test*"))
-            #     for test_folder in test_folders:
-            #         shutil.rmtree(test_folder)
+            from ucode.services.testcase.testcase_service import TestcaseService
+            TestcaseService.save_testcases(problem_folder, problem, format="ucode")
             if save_testcases_only:
                 CLog.warn("DON'T FORGET to change the 'testcase_format' to 'ucode' in meta data file.")
 
-            problem.testcases_sample = sample_testcases
-
         problem_folder = os.path.abspath(problem_folder)
 
         # copy resource files
@@ -1158,7 +1153,27 @@ class ProblemService:
         return testcases
 
 
+def convert_all_v1_to_new_format(base_folder):
+    # problems = ProblemService.read_all_problems_v1(base_folder)
+    problems = ProblemService.read_all_problems_v1(base_folder, load_testcase=True, translations=["vi"])
+
+    for folder, problem in problems:
+        print(folder)
+        base, folder_name = os.path.split(folder)
+
+        ProblemService.save(problem, base + "_new", problem_folder=folder_name, overwrite=True)
+        print(problem.name)
+        print(problem.testcase_format)
+        print(len(problem.testcases))
+        print(len(problem.testcases_sample))
+
+
 if __name__ == "__main__":
+    # base_folder = "D:\\projects\\ucode\\dsa-problems\\competitions\\hsg_thcs\\Thuc\\HaNoi2019"
+    # base_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\week10"
+    # base_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\competitions\\hsg-hn-2020"
+
+    # convert_all_v1_to_new_format(base_folder)
     # DsaProblem.create_problem('../../problems', 'Counting Sort 3', lang="en",
     #                           translations=['vi', 'ru'],
     #                           overwrite=True,
@@ -1169,14 +1184,28 @@ if __name__ == "__main__":
     #                       translations=['vi'], load_testcase=True)
     # problem = DsaProblem.load_v1('/home/thuc/projects/ucode/weekly-algorithm-problems/week03/p02_key_races',
     #                       translations=['vi'], load_testcase=True)
-    problem = ProblemService.load('../../../problems/another_one_bites_the_dust_', load_testcase=True)
-    # problem = ProblemService.load('../../../problems/another_one_bites_the_dust', load_testcase=True)
-    ProblemService.save(problem, "../../../problems", problem_folder="big_mod_py_", overwrite=True)
+
+    # folder = "../../../problems/p3_delrow"
+    folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\quan_ba_dinh_2020_2021_thcs\\cable"
+    # problem = ProblemService.load(folder, load_testcase=True)
+    problem = ProblemService.load(folder, load_testcase=True)
+    # ProblemService.save(problem, "D:\\projects\\ucode\\_problem_dev", problem_folder="big_mod_py_", overwrite=True)
+    print(json.dumps(problem.solutions))
     print(problem.name)
-    print(problem.testcase_format)
-    print(len(problem.testcases))
-    print(len(problem.stock_testcases))
-    print(len(problem.testcases_sample))
+    print("Input:")
+    print(problem.input_format)
+    print("Constraints:")
+    print(problem.constraints)
+    print("Output:")
+    print(problem.output_format)
+    print("Statement:")
+    print(problem.statement)
+
+    ProblemService.save(problem, base_folder="../../../problems/", problem_folder="_save", overwrite=True)
+    # print(problem.testcase_format)
+    # print(len(problem.testcases))
+    # print(len(problem.stock_testcases))
+    # print(len(problem.testcases_sample))
     # print("Name:", problem.testcases[0].name)
     # ProblemService.save(problem, '../../../problems',
     #                     problem_folder='another_one_bites_the_dust2', overwrite=True)
